<?php
/**
 * The template for displaying comments.
 *
 * The area of the page that contains both current comments
 * and the comment form.
 *
 * @package angllia
 */

/*
 * If the current post is protected by a password and
 * the visitor has not yet entered the password we will
 * return early without loading the comments.
 */
if ( post_password_required() ) {
	return;
}
?>
<div id="comments" class="comments-area">
<div class="comments">		
<?php if ( have_comments() ) : ?>
	
	<h3 class="title-box title-box-single"><?php comments_number(__('No Comment','angllia'), __('One Comment','angllia'), __('% Comments','angllia') );?></h3>
	<ol class="comment-list">
		<?php
			/* Loop through and list the comments. Tell wp_list_comments()
			 * to use marketify_comment() to format the comments.
			 * If you want to override this in a child theme, then you can
			 * define marketify_comment() and that will be used instead.
			 * See marketify_comment() in inc/template-tags.php for more.
			 */
			wp_list_comments( array( 'callback' => 'angllia_mytheme_comment', 'avatar_size' => 70 ) );
		?>
	</ol><!-- .comment-list -->

	<?php if ( get_comment_pages_count() > 1 && get_option( 'page_comments' ) ) : // are there comments to navigate through ?>
	<div class="rst-pagination">
		<nav class="row navigation" role="navigation">
			<div class="col-sm-6 nav-previous text-left"><?php previous_comments_link(); ?></div>
			<div class="col-sm-6 nav-next text-right"><?php next_comments_link(); ?></div>
		</nav>

		</div>
	<?php endif; // check for comment navigation ?>
<?php endif; // have_comments() ?>
</div>
<?php
function angllia_mytheme_comment($comment, $args, $depth) {
	$GLOBALS['comment'] = $comment;
	extract($args, EXTR_SKIP);

	if ( 'div' == $args['style'] ) {
		$tag = 'div';
		$add_below = 'comment';
	} else {
		$tag = 'li';
		$add_below = 'div-comment';
	}
?>
	<<?php echo $tag ?> <?php comment_class( empty( $args['has_children'] ) ? '' : 'parent' ) ?> id="comment-<?php comment_ID() ?>">
	<?php if ( 'div' != $args['style'] ) : ?>
	<div id="div-comment-<?php comment_ID() ?>" class="media comment fully-inner-box">
		<?php endif; ?>
		<div class="fully-media-left">            			
			<?php
			if ( $args['avatar_size'] != 0 ) {
				echo get_avatar( $comment, $args['avatar_size'] ); 
			}	
			?>			
		
		</div>
		<div class="fully-media-body">
			<?php if ( $comment->comment_approved == '0' ) : ?>
				<em class="comment-awaiting-moderation"><?php _e( 'Your comment is awaiting moderation.','angllia' ); ?></em>
				<br />
			<?php endif; ?>
			<h5>
				<?php
					echo esc_html($comment->comment_author);
				?>
			</h5>
			<?php comment_text(); ?>
			<span class="date-comment"><?php echo get_comment_date('d') .' . '. get_comment_date('M') .' . '. get_comment_date('Y') ?></span>
			
			<?php
				if ( is_user_logged_in() ){
			?>
			<?php comment_reply_link( array_merge( $args, array( 'add_below' => $add_below, 'reply_text' => 'Reply' ,'depth' => $depth, 'max_depth' => $args['max_depth'] ) ) ); ?>
			<?php
				}
			?>
		</div>
		<?php if ( 'div' != $args['style'] ) : ?>
	</div>
	<?php endif; ?>
<?php
}
?>
<div class="comment-form">	
	<h3 class="title-box"><?php angllia_the_translate('Leave A Comment','translation_leave_comment') ?></h4>
	<div class="box-white">
	<?php
		$commenter = wp_get_current_commenter();
		$req = get_option( 'require_name_email' );
		$aria_req = ( $req ? " aria-required='true'" : '' );
		$args = array(
			'id_form' => 'contact-form',
			'class_form' => 'form',
			'cancel_reply_link' => __( 'Cancel Reply','angllia' ),
			'fields' => apply_filters(
				'comment_form_default_fields', array(
					'author' => '<div class="form-group fully-form-box">
									<input class="form-control fully-input-form" id="author" name="author" placeholder="' . esc_html__('Your Name:','angllia') . '" type="text" value="' . esc_attr( $commenter['comment_author'] ) . '"' . $aria_req . ' />
								</div>',
					'email' => '<div class="form-group fully-form-box">
									<input class="form-control fully-input-form" id="email" name="email" placeholder="' . esc_html__('Your Email:','angllia') . '" type="text" value="' . esc_attr(  $commenter['comment_author_email'] ) . '"' . $aria_req . ' />
									
								</div>',
				)
			),
			'comment_field' => '<textarea class="form-control fully-input-form" id="comment" name="comment" placeholder="' . esc_html__('Your Comment:','angllia') . '" cols="" rows="6" aria-required="true"></textarea>',
			'comment_notes_after' => '',
			'comment_notes_before' => '<p>'.esc_html__('Your email address will not be published. Required fields are marked *','angllia').'</p>',
			'title_reply' => '',
			'format'            => 'xhtml',
			'label_submit' => angllia_get_translate('Submit','sent_comment'),
			'class_submit' => 'fully-input-submit',
		);
	  ?>
	<?php comment_form($args, get_the_ID()); ?>
	</div>
</div>

</div><!-- #comments -->
